@@ -2,13 +2,15 @@
 /**
  * @file defLex.c
  *
- *  Time-stamp:        "2011-06-03 12:20:36 bkorb"
- *
  *  This module scans the template variable declarations and passes
  *  tokens back to the parser.
  *
+ * @addtogroup autogen
+ * @{
+ */
+/*
  *  This file is part of AutoGen.
- *  AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (C) 1992-2013 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -23,8 +25,6 @@
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-static char const zErrMsg[] = "%s Error:  %s in %s on line %d\n";
-
 /*
  *  This keyword table must match those found in agParse.y.
  *  You will find them in a %token statement that follows
@@ -51,8 +51,6 @@ te_dp_event aKeywordTkn[] = { KEYWORD_TABLE };
 #define ERROR  (-1)
 #define FINISH (-1)
 
-#define SET_LIT_TKN(t) lastToken = DP_EV_LIT_ ## t; *(pCurCtx->pzScan++) = NUL;
-
 /* = = = START-STATIC-FORWARD = = = */
 static void
 pop_context(void);
@@ -69,6 +67,9 @@ lex_backquote(void);
 static tSuccess
 lex_comment(void);
 
+static tSuccess
+lex_dollar(void);
+
 static tSuccess
 lex_here_string(void);
 
@@ -79,10 +80,10 @@ static void
 alist_to_autogen_def(void);
 
 static char*
-assembleName(char * pzScan, te_dp_event * pRetVal);
+gather_name(char * pzScan, te_dp_event * pRetVal);
 
 static char*
-assembleHereString(char* pzScan);
+build_here_str(char* scan);
 /* = = = END-STATIC-FORWARD = = = */
 
 /**
@@ -91,76 +92,80 @@ assembleHereString(char* pzScan);
 static void
 pop_context(void)
 {
-    tScanCtx* pCX = pCurCtx;
-    pCurCtx   = pCurCtx->pCtx;
-    pCX->pCtx = pDoneCtx;
-    pDoneCtx  = pCX;
+    scan_ctx_t* pCX = cctx;
+    cctx          = cctx->scx_next;
+    pCX->scx_next = end_ctx;
+    end_ctx       = pCX;
 }
 
 static void
 trim_whitespace(void)
 {
-    char* pz = pCurCtx->pzScan;
-    if (*pz == NL)
-        pCurCtx->lineNo++;
-    *(pz++) = NUL;
+    char * pz = cctx->scx_scan;
 
     /*
      *  This ensures that any names found previously
      *  are NUL terminated.
      */
-    while (IS_WHITESPACE_CHAR(*pz)) {
-        if (*pz == NL)
-            pCurCtx->lineNo++;
-        pz++;
+    if (*pz == NL)
+        cctx->scx_line++;
+    *pz = NUL;
+
+    for (;;) {
+        pz = SPN_HORIZ_WHITE_CHARS(pz+1);
+        if (*pz != NL)
+            break;
+        cctx->scx_line++;
     }
-    pCurCtx->pzScan = pz;
+
+    cctx->scx_scan = pz;
 }
 
 static void
 lex_escaped_char(void)
 {
-    char* pz = strchr(pCurCtx->pzScan, ';');
+    static int const semi_colon = ';';
+
+    char * pz = strchr(cctx->scx_scan, semi_colon);
 
     for (;;) {
         if (pz == NULL) {
-            pz = pCurCtx->pzScan + strlen(pCurCtx->pzScan);
+            pz = cctx->scx_scan + strlen(cctx->scx_scan);
             break;
         }
         if (IS_WHITESPACE_CHAR(pz[1])) {
             *pz = NUL;
-            pz[1] = ';';
+            pz[1] = (char)semi_colon;
             break;
         }
-        pz = strchr(pz+1, ';');
+        pz = strchr(pz+1, semi_colon);
     }
 
-    lastToken = DP_EV_STRING;
-    pz_token = pz;
+    token_code = DP_EV_STRING;
+    token_str = pz;
 }
 
 static tSuccess
 lex_backquote(void)
 {
-    int line_no = pCurCtx->lineNo;
-    char* pz = ao_string_cook(pCurCtx->pzScan, &line_no);
+    int line_no = cctx->scx_line;
+    char* pz = ao_string_cook(cctx->scx_scan, &line_no);
 
     if (pz == NULL)
         return FAILURE;
 
-    pz_token = pCurCtx->pzScan;
+    token_str = cctx->scx_scan;
 
-    pCurCtx->pzScan = pz;
+    cctx->scx_scan = pz;
 
-    lastToken = DP_EV_STRING;
-    pz = shell_cmd((char const*)pz_token);
-    pCurCtx->lineNo = line_no;
+    token_code = DP_EV_STRING;
+    pz = shell_cmd((char const*)token_str);
+    cctx->scx_line = line_no;
 
     if (pz == NULL)
         return PROBLEM;
-    TAGMEM(pz, "shell definition string");
-    pz_token = pz;
-    manageAllocatedData(pz);
+
+    token_str = pz;
     return SUCCESS;
 }
 
@@ -170,29 +175,29 @@ lex_comment(void)
     /*
      *  Allow for a comment, C or C++ style
      */
-    switch (pCurCtx->pzScan[1]) {
+    switch (cctx->scx_scan[1]) {
     case '*':
     {
-        char* pz = strstr(pCurCtx->pzScan+2, "*/");
+        char* pz = strstr(cctx->scx_scan+2, END_C_COMMENT);
         if (pz != NULL) {
-            char* p = pCurCtx->pzScan+1;
+            char* p = cctx->scx_scan+1;
             for (;;) {
                 p = strchr(p+1, NL);
                 if ((p == NULL) || (p > pz))
                     break;
-                pCurCtx->lineNo++;
+                cctx->scx_line++;
             }
-            pCurCtx->pzScan = pz+2;
+            cctx->scx_scan = pz+2;
             return SUCCESS;
         }
         break;
     }
     case '/':
     {
-        char* pz = strchr(pCurCtx->pzScan+2, NL);
+        char* pz = strchr(cctx->scx_scan+2, NL);
         if (pz != NULL) {
-            pCurCtx->pzScan = pz+1;
-            pCurCtx->lineNo++;
+            cctx->scx_scan = pz+1;
+            cctx->scx_line++;
             return SUCCESS;
         }
         break;
@@ -202,21 +207,75 @@ lex_comment(void)
     return FAILURE;
 }
 
+/**
+ * process a dollar character introduced value.
+ *
+ * '$(<...)' load a file that must be there.
+ * '$(?...)' load a file that may be there. If not there, then delete
+ *           the named value associated with it.
+ */
+static tSuccess
+lex_dollar(void)
+{
+    char * next;
+
+    /*
+     * First, ensure the string starts with what we recognize
+     */
+    if (cctx->scx_scan[1] != '(')
+        return FAILURE;
+    switch (cctx->scx_scan[2]) {
+    case '<':
+    case '?': break;
+    default: return FAILURE;
+    }
+
+    /*
+     * Find the start and end of the file name and load the file text.
+     */
+    do {
+        char * fns = SPN_WHITESPACE_CHARS(cctx->scx_scan+3);
+        char * fne = strchr(fns, ')');
+        char   svc;
+
+        if (fne == NULL)
+            return FAILURE;
+        next = fne + 1;
+        fne  = SPN_WHITESPACE_BACK(fns, fne-1);
+        svc  = *fne;
+        *fne = NUL;
+        token_str = load_file(fns);
+        *fne = svc;
+        if (token_str == NULL)
+            break;
+        token_code = DP_EV_STRING;
+        cctx->scx_scan = next;
+        return SUCCESS; /* done -- we've loaded the file. */
+    } while (0);
+
+    if (cctx->scx_scan[2] == '<')
+        return FAILURE;
+
+    cctx->scx_scan = next;
+    token_code = DP_EV_DELETE_ENT;
+    return SUCCESS;
+}
+
 static tSuccess
 lex_here_string(void)
 {
     char* pz;
-    if (pCurCtx->pzScan[1] != '<')
+    if (cctx->scx_scan[1] != '<')
         return FAILURE;
 
-    pz = assembleHereString(pCurCtx->pzScan + 2);
+    pz = build_here_str(cctx->scx_scan + 2);
     if (pz == NULL) {
-        lastToken = DP_EV_INVALID;
+        token_code = DP_EV_INVALID;
         return PROBLEM;
     }
 
-    lastToken = DP_EV_HERE_STRING;
-    pCurCtx->pzScan = pz;
+    token_code = DP_EV_HERE_STRING;
+    cctx->scx_scan = pz;
     return SUCCESS;
 }
 
@@ -227,39 +286,42 @@ lex_here_string(void)
 LOCAL te_dp_event
 yylex(void)
 {
-    lastToken = DP_EV_INVALID;
+#define SET_LIT_TKN(t) \
+    token_code = DP_EV_LIT_ ## t; *(cctx->scx_scan++) = NUL;
 
-scanAgain:
+    token_code = DP_EV_INVALID;
+
+scan_again:
     /*
      *  Start the process of locating a token.
      *  We branch here after skipping over a comment
      *  or processing a directive (which may change our context).
      */
-    if (IS_WHITESPACE_CHAR(*pCurCtx->pzScan))
+    if (IS_WHITESPACE_CHAR(*cctx->scx_scan))
         trim_whitespace();
 
-    switch (*pCurCtx->pzScan) {
+    switch (*cctx->scx_scan) {
     case NUL:
         /*
          *  IF we are not inside an include context,
          *  THEN go finish.
          */
-        if (pCurCtx->pCtx == NULL)
+        if (cctx->scx_next == NULL)
             goto lex_done;
 
         pop_context();
-        goto scanAgain;
+        goto scan_again;
 
     case '#':
     {
-        extern char * processDirective(char*);
-        char * pz = processDirective(pCurCtx->pzScan+1);
+        extern char * processDirective(char *);
+        char * pz = processDirective(cctx->scx_scan+1);
         /*
          *  Ensure that the compiler doesn't try to save a copy of
-         *  "pCurCtx" in a register.  It must be reloaded from memory.
+         *  "cctx" in a register.  It must be reloaded from memory.
          */
-        pCurCtx->pzScan = pz;
-        goto scanAgain;
+        cctx->scx_scan = pz;
+        goto scan_again;
     }
 
     case '{': SET_LIT_TKN(O_BRACE);   break;
@@ -273,21 +335,26 @@ scanAgain:
     case '\'':
     case '"':
     {
-        char* pz = ao_string_cook(pCurCtx->pzScan, &(pCurCtx->lineNo));
+        char* pz = ao_string_cook(cctx->scx_scan, &(cctx->scx_line));
         if (pz == NULL)
             goto NUL_error;
 
-        pz_token = pCurCtx->pzScan;
+        token_str = cctx->scx_scan;
 
-        lastToken = DP_EV_STRING;
-        pCurCtx->pzScan = pz;
+        token_code = DP_EV_STRING;
+        cctx->scx_scan = pz;
         break;
     }
 
+    case '$':
+        if (lex_dollar() == SUCCESS)
+            break;
+        goto bad_token;
+        
     case '<':
         switch (lex_here_string()) {
         case SUCCESS: break;
-        case FAILURE: goto BrokenToken;
+        case FAILURE: goto bad_token;
         case PROBLEM: return DP_EV_INVALID;
         }
         break;
@@ -297,9 +364,9 @@ scanAgain:
         break;
 
     case '\\':
-        if (strncmp(pCurCtx->pzScan+1, "'(", (size_t)2) == 0) {
+        if (strncmp(cctx->scx_scan+1, START_SCHEME_LIST, (size_t)2) == 0) {
             alist_to_autogen_def();
-            goto scanAgain;
+            goto scan_again;
         }
         lex_escaped_char();
         break;
@@ -307,93 +374,82 @@ scanAgain:
     case '`':
         switch (lex_backquote()) {
         case FAILURE: goto NUL_error;
-        case PROBLEM: goto scanAgain;
+        case PROBLEM: goto scan_again;
         case SUCCESS: break;
         }
         break;
 
     case '/':
-        switch (lex_comment()) {
-        case SUCCESS: goto scanAgain;
-        default: break;
-        }
+        if (lex_comment() == SUCCESS)
+            goto scan_again;
         /* FALLTHROUGH */ /* to Invalid input char */
 
     default:
-    BrokenToken:
-        pCurCtx->pzScan = assembleName(pCurCtx->pzScan, &lastToken);
+    bad_token:
+        cctx->scx_scan = gather_name(cctx->scx_scan, &token_code);
         break;
-    }   /* switch (*pCurCtx->pzScan) */
+    }   /* switch (*cctx->scx_scan) */
 
-    return lastToken;
+    return token_code;
 
-NUL_error:
+ NUL_error:
 
-    AG_ABEND(aprf(zErrMsg, pzProg, "unterminated quote in definition",
-                  pCurCtx->pzCtxFname, pCurCtx->lineNo));
+    AG_ABEND(aprf(DEF_ERR_FMT, ag_pname, YYLEX_UNENDING_QUOTE,
+                  cctx->scx_fname, cctx->scx_line));
     return DP_EV_INVALID;
 
-lex_done:
+ lex_done:
     /*
      *  First time through, return the DP_EV_END token.
      *  Second time through, we really finish.
      */
-    if (pCurCtx->pzScan == zNil) {
-        pCurCtx->pCtx = pDoneCtx;
-        pDoneCtx      = pCurCtx;
+    if (cctx->scx_scan == zNil) {
+        cctx->scx_next = end_ctx;
+        end_ctx        = cctx;
 
         return DP_EV_INVALID;
     }
 
-    pCurCtx->pzScan = (char*)zNil;
+    cctx->scx_scan = (char *)zNil;
     return DP_EV_END;
+#undef SET_LIT_TKN
 }
 
 
 LOCAL void
 yyerror(char* s)
 {
-    static char const yyerr_fmt[] =
-        "%s:  in %s on line %d\n"
-        "\ttoken in error:  %s\n"
-        "\t[[...<error-text>]] %s\n\n"
-        "Likely causes:  a mismatched quote, a value that needs "
-        "quoting,\n\t\tor a missing semi-colon\n";
+    char * pz;
 
-    static char const zErrTkn[] = "%s:  ``%s''\n";
-    static char const zDf[] = "`%s'\n";
+    if (strlen(cctx->scx_scan) > 64 )
+        cctx->scx_scan[64] = NUL;
 
-    char* pz;
-
-    if (strlen(pCurCtx->pzScan) > 64 )
-        pCurCtx->pzScan[64] = NUL;
-
-    switch (lastToken) {
+    switch (token_code) {
     case DP_EV_VAR_NAME:
     case DP_EV_OTHER_NAME:
     case DP_EV_STRING:
     case DP_EV_NUMBER:
-        if (strlen(pz_token) > 64 )
-            pz_token[64] = NUL;
+        if (strlen(token_str) > 64 )
+            token_str[64] = NUL;
 
-        pz = aprf(zErrTkn, DP_EVT_NAME(lastToken), pz_token);
+        pz = aprf(YYLEX_TOKEN_STR, DP_EVT_NAME(token_code), token_str);
         break;
 
     default:
-        pz = aprf(zDf, DP_EVT_NAME(lastToken));
+        pz = aprf(YYLEX_DF_STR, DP_EVT_NAME(token_code));
     }
-    AG_ABEND(aprf(yyerr_fmt, s, pCurCtx->pzCtxFname, pCurCtx->lineNo, pz,
-                  pCurCtx->pzScan));
+    AG_ABEND(aprf(YYLEX_ERR_FMT, s, cctx->scx_fname, cctx->scx_line,
+                  pz, cctx->scx_scan));
 }
 
 
 static void
 loadScheme(void)
 {
-    char*    pzText    = pCurCtx->pzScan;
-    char*    pzEnd     = (char*)skipScheme(pzText, pzText + strlen(pzText));
+    char*    pzText    = cctx->scx_scan;
+    char*    pzEnd     = (char *)skip_scheme(pzText, pzText + strlen(pzText));
     char     endCh     = *pzEnd;
-    int      schemeLen = (pzEnd - pzText);
+    int      schemeLen = (int)(pzEnd - pzText);
     int      next_ln;
     SCM      res;
 
@@ -402,28 +458,26 @@ loadScheme(void)
      *  the NUL-ed character.
      */
     if (*pzEnd == NUL)
-        AG_ABEND(aprf(zErrMsg, pzProg,
-                      "end of Guile/scheme expression not found",
-                      pCurCtx->pzCtxFname, pCurCtx->lineNo));
+        AG_ABEND(aprf(DEF_ERR_FMT, ag_pname, LOAD_SCM_ENDLESS,
+                      cctx->scx_fname, cctx->scx_line));
 
     *pzEnd  = NUL;
-    next_ln = pCurCtx->lineNo + count_nl(pzText);
+    next_ln = cctx->scx_line + count_nl(pzText);
 
-    procState = PROC_STATE_GUILE_PRELOAD;
+    processing_state = PROC_STATE_GUILE_PRELOAD;
     res = ag_scm_c_eval_string_from_file_line(
-        pzText, pCurCtx->pzCtxFname, pCurCtx->lineNo );
-    procState = PROC_STATE_LOAD_DEFS;
+        pzText, cctx->scx_fname, cctx->scx_line );
+    processing_state = PROC_STATE_LOAD_DEFS;
     *pzEnd = endCh;
 
-    pCurCtx->pzScan = pzEnd;
-    pzEnd = (char*)resolveSCM(res); /* ignore const-ness */
-    pCurCtx->lineNo = next_ln;
+    cctx->scx_scan = pzEnd;
+    pzEnd = (char *)scm2display(res); /* ignore const-ness */
+    cctx->scx_line = next_ln;
 
-    if (strlen(pzEnd) >= schemeLen) {
+    if ((int)strlen(pzEnd) >= schemeLen) {
         AGDUPSTR(pzEnd, pzEnd, "SCM Result");
 
-        pz_token = pzEnd;
-        manageAllocatedData(pz_token);
+        token_str = pzEnd;
     }
 
     else {
@@ -431,10 +485,10 @@ loadScheme(void)
          *  We know the result is smaller than the source.  Copy in place.
          */
         strcpy(pzText, pzEnd);
-        pz_token = pzText;
+        token_str = pzText;
     }
 
-    lastToken = DP_EV_STRING;
+    token_code = DP_EV_STRING;
 }
 
 /*
@@ -444,15 +498,12 @@ loadScheme(void)
 static void
 alist_to_autogen_def(void)
 {
-    static char const zSchemeText[] = "Scheme Computed Definitions";
-    static char const zWrap[] = "(alist->autogen-def %s)";
-
-    char*  pzText  = ++(pCurCtx->pzScan);
-    char*  pzEnd   = (char*)skipScheme(pzText, pzText + strlen(pzText));
+    char*  pzText  = ++(cctx->scx_scan);
+    char*  pzEnd   = (char *)skip_scheme(pzText, pzText + strlen(pzText));
 
     SCM    res;
     size_t res_len;
-    tScanCtx*  pCtx;
+    scan_ctx_t*  pCtx;
 
     /*
      *  Wrap the scheme expression with the `alist->autogen-def' function
@@ -460,54 +511,51 @@ alist_to_autogen_def(void)
     {
         char endCh = *pzEnd;
         *pzEnd = NUL;
-        pzText = aprf(zWrap, pzText);
+        pzText = aprf(ALIST_TO_AG_WRAP, pzText);
         *pzEnd = endCh;
     }
 
     /*
      *  Run the scheme expression.  The result is autogen definition text.
      */
-    procState = PROC_STATE_GUILE_PRELOAD;
+    processing_state = PROC_STATE_GUILE_PRELOAD;
     res = ag_scm_c_eval_string_from_file_line(
-        pzText, pCurCtx->pzCtxFname, pCurCtx->lineNo );
+        pzText, cctx->scx_fname, cctx->scx_line );
 
     /*
      *  The result *must* be a string, or we choke.
      */
-    if (! AG_SCM_STRING_P(res)) {
-        static char const zEr[] =
-            "Scheme definition expression does not yield string:\n";
-        AG_ABEND(zEr);
-    }
+    if (! AG_SCM_STRING_P(res))
+        AG_ABEND(ALIST_TO_AG_ERR);
 
     res_len   = AG_SCM_STRLEN(res);
-    procState = PROC_STATE_LOAD_DEFS;
-    pCurCtx->pzScan = pzEnd;
+    processing_state = PROC_STATE_LOAD_DEFS;
+    cctx->scx_scan = pzEnd;
     AGFREE(pzText);
 
     /*
      *  Now, push the resulting string onto the input stack
      *  and link the new scan data into the context stack
      */
-    pCtx = (tScanCtx*)AGALOC(sizeof(tScanCtx) + 4 + res_len, "lex scan ctx");
-    pCtx->pCtx  = pCurCtx;
-    pCurCtx     = pCtx;
+    pCtx = (scan_ctx_t*)AGALOC(sizeof(scan_ctx_t) + 4 + res_len, "lex ctx");
+    pCtx->scx_next = cctx;
+    cctx           = pCtx;
 
     /*
      *  Set up the rest of the context structure
      */
-    AGDUPSTR(pCtx->pzCtxFname, zSchemeText, "scheme text");
-    pCtx->pzScan = \
-    pCtx->pzData = (char*)(pCtx+1);
-    pCtx->lineNo = 0;
-    memcpy((void*)(pCtx->pzScan), (void*)AG_SCM_CHARS(res), res_len);
-    pCtx->pzScan[ res_len ] = NUL;
+    AGDUPSTR(pCtx->scx_fname, ALIST_TO_AG_TEXT, "scheme text");
+    pCtx->scx_scan = \
+    pCtx->scx_data = (char *)(pCtx + 1);
+    pCtx->scx_line = 0;
+    memcpy((void *)(pCtx->scx_scan), (void *)AG_SCM_CHARS(res), res_len);
+    pCtx->scx_scan[ res_len ] = NUL;
 
     /*
-     *  At this point, the next token will be obtained
-     *  from the newly allocated context structure.
-     *  When empty, input will resume from the '}' that we
-     *  left as the next input token in the old context.
+     *  At this point, the next token will be obtained from the newly
+     *  allocated context structure.  When empty, input will resume
+     *  from the '}' that we left as the next input token in the old
+     *  context.
      */
 }
 
@@ -518,7 +566,7 @@ alist_to_autogen_def(void)
  *  Figure out which.
  */
 static char*
-assembleName(char * pzScan, te_dp_event * pRetVal)
+gather_name(char * pzScan, te_dp_event * pRetVal)
 {
     /*
      *  Check for a number.
@@ -528,24 +576,23 @@ assembleName(char * pzScan, te_dp_event * pRetVal)
        || (  (*pzScan == '-')
           && IS_DEC_DIGIT_CHAR(pzScan[1])
        )  )  {
-        pz_token = pzScan;
+        token_str = pzScan;
         (void)strtol(pzScan, &pzScan, 0);
         *pRetVal = DP_EV_NUMBER;
         return pzScan;
     }
 
     if (! IS_UNQUOTABLE_CHAR(*pzScan))
-        AG_ABEND(aprf("%s Error: Invalid input char '%c' in %s on line %d\n",
-                      pzProg, *pzScan, pCurCtx->pzCtxFname, pCurCtx->lineNo));
+        AG_ABEND(aprf(ASSEMBLE_NAME_ERR, ag_pname, *pzScan, cctx->scx_fname,
+                      cctx->scx_line));
 
     {
-        unsigned char* pz = (unsigned char*)pzScan;
-
-        while (IS_VALUE_NAME_CHAR(*pz))          pz++;
+        char* pz = SPN_VALUE_NAME_CHARS(pzScan);
 
         if (IS_UNQUOTABLE_CHAR(*pz)) {
             *pRetVal = DP_EV_OTHER_NAME;
-            while (IS_UNQUOTABLE_CHAR(*++pz))    ;
+            pz = SPN_UNQUOTABLE_CHARS(pz+1);
+
         } else
             *pRetVal = DP_EV_VAR_NAME;
 
@@ -554,7 +601,7 @@ assembleName(char * pzScan, te_dp_event * pRetVal)
          *  If the name is actually a keyword,
          *  we will return that token code instead.
          */
-        pz_token = pzScan;
+        token_str = pzScan;
         pzScan   = (char*)pz;
     }
 
@@ -567,14 +614,14 @@ assembleName(char * pzScan, te_dp_event * pRetVal)
         *pzScan = NUL;         /* NUL terminate the name           */
 
         do  {
-            if (streqvcmp(apzKeywords[ kw_ix ], (char*)pz_token) == 0) {
+            if (streqvcmp(apzKeywords[ kw_ix ], (char*)token_str) == 0) {
                 /*
                  *  Return the keyword token code instead of DP_EV_NAME
                  */
                 *pRetVal = aKeywordTkn[ kw_ix ];
                 break;
             }
-        } while (++kw_ix < KEYWORD_CT);
+        } while (++kw_ix < (int)KEYWORD_CT);
 
         *pzScan = sv_ch;         /* restore the following character  */
     }
@@ -582,112 +629,111 @@ assembleName(char * pzScan, te_dp_event * pRetVal)
     return pzScan;
 }
 
-
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
  *  A quoted string has been found.
  *  Find the end of it and compress any escape sequences.
  */
 static char*
-assembleHereString(char* pzScan)
+build_here_str(char* scan)
 {
-    static char const endless[] = "Unterminated HereString";
-    ag_bool  trimTabs = AG_FALSE;
+    bool     no_tabs = false;
     char     zMark[ MAX_HEREMARK_LEN ];
     size_t   markLen = 0;
-    char*    pzDest;
-    int      here_string_line_no;
+    char*    dest;
+    int      here_str_line;
 
     /*
      *  See if we are to strip leading tab chars
      */
-    if (*pzScan == '-') {
-        trimTabs = AG_TRUE;
-        pzScan++;
+    if (*scan == '-') {
+        no_tabs = true;
+        scan++;
     }
 
     /*
      *  Skip white space up to the marker or EOL
      */
-    while (IS_WHITESPACE_CHAR(*pzScan)) {
-        if (*pzScan++ == NL)
-            AG_ABEND(aprf(zErrMsg, pzProg, "HereString missing the mark",
-                          pCurCtx->pzCtxFname, pCurCtx->lineNo));
-    }
+    scan = SPN_NON_NL_WHITE_CHARS(scan);
+    if (*scan == NL)
+        AG_ABEND(aprf(DEF_ERR_FMT, ag_pname, HERE_MISS_MARK_STR,
+                      cctx->scx_fname, cctx->scx_line));
 
     /*
      *  Copy the marker, noting its length
      */
     {
-        char* pz = zMark;
-        while (IS_VARIABLE_NAME_CHAR(*pzScan)) {
-            if (++markLen >= sizeof(zMark))
-                AG_ABEND(aprf(zErrMsg, pzProg, "HereString mark "
-                              STR(MAX_HEREMARK_LEN) " or more chars",
-                              pCurCtx->pzCtxFname, pCurCtx->lineNo));
+        char * pz = SPN_VARIABLE_NAME_CHARS(scan);
+        markLen = (size_t)(pz - scan);
 
-            *(pz++) = *(pzScan++);
-        }
         if (markLen == 0)
-            AG_ABEND(aprf(zErrMsg, pzProg, "HereString missing the mark",
-                          pCurCtx->pzCtxFname, pCurCtx->lineNo));
-        *pz = NUL;
+            AG_ABEND(aprf(DEF_ERR_FMT, ag_pname, HERE_MISS_MARK_STR,
+                          cctx->scx_fname, cctx->scx_line));
+
+        if (markLen >= sizeof(zMark))
+            AG_ABEND(aprf(DEF_ERR_FMT, ag_pname, HERE_MARK_TOO_LONG,
+                          cctx->scx_fname, cctx->scx_line));
+
+        memcpy(zMark, scan, markLen);
+        zMark[markLen] = NUL;
     }
 
-    pzDest = pzScan;
-    pz_token = pzDest;
+    token_str = dest = scan;
 
     /*
      *  Skip forward to the EOL after the marker.
      */
-    pzScan = strchr(pzScan, NL);
-    if (pzScan == NULL)
-        AG_ABEND(aprf(zErrMsg, pzProg, endless, pCurCtx->pzCtxFname,
-                      pCurCtx->lineNo));
+    scan = strchr(scan, NL);
+    if (scan == NULL)
+        AG_ABEND(aprf(DEF_ERR_FMT, ag_pname, HERE_ENDLESS_STR,
+                      cctx->scx_fname, cctx->scx_line));
 
     /*
      *  And skip the first new line + conditionally skip tabs
      */
-    here_string_line_no = pCurCtx->lineNo++;
-    pzScan++;
+    here_str_line = cctx->scx_line++;
+    scan++;
 
-    if (trimTabs)
-        while (*pzScan == TAB)  ++pzScan;
+    for (;;) {
+    next_line:
+        if (no_tabs) {
+            while (*scan == TAB)  ++scan;
+            if ((scan[0] == '\\') && (scan[1] == TAB))
+                ++scan;
+        }
 
-    /*
-     *  FOR as long as the text does not match the mark
-     *       OR it matches but is a substring
-     *  DO copy characters
-     */
-    while (  (strncmp(pzScan, zMark, markLen) != 0)
-          || IS_VARIABLE_NAME_CHAR(pzScan[markLen]) )  {
+        /*
+         *  If we recognize the end mark, break out.
+         */
+        if (! IS_VARIABLE_NAME_CHAR(scan[markLen]))
+            if (strncmp(scan, zMark, markLen) == 0)
+                break;
 
         for (;;) {
-            switch (*(pzDest++) = *(pzScan++)) {
+            switch (*(dest++) = *(scan++)) {
             case NL:
-                pCurCtx->lineNo++;
-                goto lineDone;
+                cctx->scx_line++;
+                goto next_line;
 
             case NUL:
-                AG_ABEND(aprf(zErrMsg, pzProg, endless, pCurCtx->pzCtxFname,
-                              here_string_line_no));
+                AG_ABEND(aprf(DEF_ERR_FMT, ag_pname, HERE_ENDLESS_STR,
+                              cctx->scx_fname, here_str_line));
             }
-        } lineDone:;
-
-        if (trimTabs)
-            while (*pzScan == TAB)  ++pzScan;
+        }
     } /* while strncmp ... */
 
     /*
-     *  pzDest may still equal pz_token, if no data were copied
+     *  dest may still equal token_str, if no data were copied
      */
-    if (pzDest > (char*)pz_token)
-         pzDest[-1] = NUL;
-    else pzDest[0]  = NUL;
+    if (dest > (char*)token_str)
+         dest[-1] = NUL;
+    else dest[0]  = NUL;
 
-    return pzScan + markLen;
+    return scan + markLen;
 }
-/*
+/**
+ * @}
+ *
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
