@@ -2,12 +2,14 @@
 /**
  * @file functions.c
  *
- *  Time-stamp:        "2012-06-10 11:24:45 bkorb"
- *
  *  This module implements text functions.
  *
+ * @addtogroup autogen
+ * @{
+ */
+/*
  *  This file is part of AutoGen.
- *  AutoGen Copyright (c) 1992-2012 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (C) 1992-2013 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -158,7 +160,7 @@ gen_new_block(templ_t * tpl)
  *
  *  @returns the macro table entry after mac
  */
-LOCAL macro_t *
+macro_t *
 mLoad_Leave(templ_t * tpl, macro_t * mac, char const ** p_scan)
 {
     (void) tpl;
@@ -390,13 +392,13 @@ macro_t *
 mLoad_Unknown(templ_t * tpl, macro_t * mac, char const ** unused)
 {
     char const * scan;
-    ssize_t      src_len = (size_t)mac->md_res; /* macro len  */
+    ssize_t      src_len = (ssize_t)mac->md_res; /* macro len  */
     (void)unused;
 
     if (src_len <= 0)
         goto return_emtpy_expr;
 
-    scan = (char const*)mac->md_txt_off; /* macro text */
+    scan = (char const *)mac->md_txt_off; /* macro text */
 
     switch (*scan) {
     case ';':
@@ -410,11 +412,17 @@ mLoad_Unknown(templ_t * tpl, macro_t * mac, char const ** unused)
             scan = strchr(scan, NL);
             if (scan == NULL)
                 goto return_emtpy_expr;
+
             scan = SPN_WHITESPACE_CHARS(scan);
             if (*scan == NUL)
                 goto return_emtpy_expr;
+
+            src_len -= scan - start;
+            if (src_len <= 0)
+                goto return_emtpy_expr;
+
+            start = scan;
         } while (*scan == ';');
-        src_len -= scan - start;
         break;
     }
 
@@ -436,19 +444,19 @@ mLoad_Unknown(templ_t * tpl, macro_t * mac, char const ** unused)
          */
         while (IS_WHITESPACE_CHAR(scan[-1])) scan--, src_len++;
         scan    -= cname_len;
-        src_len += cname_len;
+        src_len += (ssize_t)cname_len;
 
         /*
          *  Now copy over the full canonical name.  Check for errors.
          *  Advance the scan pointer to just past the name we've copied.
          */
         {
-            size_t rem_len = canonical_name(cname, scan, (int)src_len);
+            ssize_t rem_len = canonical_name(cname, scan, (int)src_len);
             if (rem_len > src_len)
                 AG_ABEND_IN(tpl, mac, LD_UNKNOWN_INVAL_DEF);
 
             scan   += src_len - rem_len;
-            src_len = rem_len;
+            src_len = (ssize_t)rem_len;
         }
 
         /*
@@ -467,9 +475,9 @@ mLoad_Unknown(templ_t * tpl, macro_t * mac, char const ** unused)
      */
     {
         char * dest = tpl->td_scan; /* next text dest   */
-        mac->md_txt_off = (dest - tpl->td_text);
+        mac->md_txt_off = (uintptr_t)(dest - tpl->td_text);
         mac->md_res = 0;
-        memcpy(dest, scan, src_len);
+        memcpy(dest, scan, (size_t)src_len);
         dest       += src_len;
         *(dest++)   = NUL;
         *dest       = NUL; /* double terminate */
@@ -530,7 +538,9 @@ mLoad_Bogus(templ_t * tpl, macro_t * mac, char const ** p_scan)
     /* NOTREACHED */
     return NULL;
 }
-/*
+/**
+ * @}
+ *
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
