@@ -1,300 +1,299 @@
 /* -*- buffer-read-only: t -*- vi: set ro:
  *
  * Prototypes for agen5
- * Generated Sat Aug 11 09:41:33 PDT 2012
+ * Generated Sun Mar 31 10:41:50 PDT 2013
  */
 #ifndef AGEN5_PROTO_H_GUARD
 #define AGEN5_PROTO_H_GUARD 1
 
-#ifndef LOCAL
-#  define LOCAL extern
-#  define REDEF_LOCAL 1
-#else
-#  undef  REDEF_LOCAL
-#endif
 /*
  *  Extracted from agCgi.c
  */
-LOCAL void
+static void
 loadCgi(void);
 
 /*
  *  Extracted from agDep.c
  */
-LOCAL void
+static void
 add_source_file(char const * pz);
 
-LOCAL void
+static void
 rm_source_file(char const * pz);
 
-LOCAL void
+static void
 add_target_file(char const * pz);
 
-LOCAL void
+static void
 rm_target_file(char const * pz);
 
-LOCAL void
+static void
 start_dep_file(void);
 
-LOCAL void
+static void
 wrap_up_depends(void);
 
 /*
  *  Extracted from agInit.c
  */
-LOCAL void
-initialize(int arg_ct, char** arg_vec);
+static void
+initialize(int arg_ct, char ** arg_vec);
 
-LOCAL void
+static void
 config_dep(tOptions * opts, tOptDesc * od);
 
+static void
+prep_env(void);
+
 /*
  *  Extracted from agShell.c
  */
-LOCAL void
+static void
 close_server_shell(void);
 
-LOCAL char *
+static char *
 shell_cmd(char const * cmd);
 
 /*
  *  Extracted from agUtils.c
  */
-LOCAL char *
+static char *
 aprf(char const * pzFmt, ...);
 
-LOCAL void
+static void
 open_trace_file(char ** av, tOptDesc * odsc);
 
-LOCAL void
+static void
 check_make_dep_env(void);
 
-LOCAL void
+static void
 process_ag_opts(int arg_ct, char ** arg_vec);
 
-LOCAL char const *
+static char const *
 get_define_str(char const * de_name, bool check_env);
 
-LOCAL char *
+static char *
 span_quote(char * in_q);
 
-LOCAL char const *
+static char const *
 skip_scheme(char const * scan,  char const * end);
 
-LOCAL int
+static int
 count_nl(char const * pz);
 
-LOCAL char const *
+static char const *
 skip_expr(char const * pzSrc, size_t len);
 
 /*
  *  Extracted from autogen.c
  */
-LOCAL void
+static void
 ag_abend_at(char const * pzMsg
 #ifdef DEBUG_ENABLED
             , char const * pzFile, int line
 #endif
     );
 
-LOCAL void *
+static void *
 ao_malloc (size_t sz);
 
-LOCAL void *
+static void *
 ao_realloc (void *p, size_t sz);
 
-LOCAL char *
+static char *
 ao_strdup (char const * str);
 
 /*
  *  Extracted from defDirect.c
  */
-LOCAL char*
-processDirective(char* pzScan);
+static char *
+processDirective(char * scan);
 
 /*
  *  Extracted from defFind.c
  */
-LOCAL int
+static int
 canonical_name(char * pzD, char const * pzS, int srcLen);
 
-LOCAL def_ent_t *
+static def_ent_t *
 find_def_ent(char * name, bool * indexed);
 
-LOCAL void
+static void
 print_used_defines(void);
 
-LOCAL def_ent_t **
+static def_ent_t **
 find_def_ent_list(char * name);
 
 /*
  *  Extracted from defLex.c
  */
-LOCAL te_dp_event
+static te_dp_event
 yylex(void);
 
-LOCAL void
+static void
 yyerror(char* s);
 
 /*
  *  Extracted from defLoad.c
  */
-LOCAL def_ent_t *
+static def_ent_t *
 new_def_ent(void);
 
-LOCAL void
-print_ent(def_ent_t * pDef);
+static void
+print_ent(def_ent_t * de);
 
-LOCAL def_ent_t *
+static void
+delete_ent(def_ent_t * de);
+
+static def_ent_t *
 number_and_insert_ent(char * name, char const * idx_str);
 
-LOCAL void
+static void
 read_defs(void);
 
-LOCAL void
+static void
 unload_defs(void);
 
+/*
+ *  Extracted from expExtract.c
+ */
+static char *
+load_file(char const * fname);
+
 /*
  *  Extracted from expGuile.c
  */
-LOCAL teGuileType
+static teGuileType
 ag_scm_type_e(SCM typ);
 
-LOCAL SCM
+static SCM
 ag_scm_c_eval_string_from_file_line(
     char const * pzExpr, char const * pzFile, int line);
 
 /*
  *  Extracted from expOutput.c
  */
-LOCAL void
+static void
 make_readonly(void);
 
-LOCAL void
+static void
 open_output_file(char const * fname, size_t nmsz, char const * mode, int flags);
 
 /*
  *  Extracted from expPrint.c
  */
-LOCAL SCM
+static SCM
 run_printf(char const * pzFmt, int len, SCM alist);
 
 /*
  *  Extracted from expString.c
  */
-LOCAL void
+static void
 do_multi_subs(char ** ppzStr, ssize_t * pStrLen, SCM match, SCM repl);
 
 /*
  *  Extracted from funcDef.c
  */
-LOCAL void
+static void
 parse_mac_args(templ_t * pT, macro_t * mac);
 
 /*
  *  Extracted from funcEval.c
  */
-LOCAL char const *
+static char const *
 scm2display(SCM s);
 
-LOCAL char const *
+static char const *
 eval_mac_expr(bool * allocated);
 
-LOCAL SCM
+static SCM
 eval(char const * expr);
 
 /*
  *  Extracted from funcFor.c
  */
-LOCAL for_state_t *
+static for_state_t *
 new_for_context(void);
 
-LOCAL void
+static void
 free_for_context(bool everything);
 
 /*
  *  Extracted from functions.c
  */
-LOCAL loop_jmp_type_t
+static loop_jmp_type_t
 call_gen_block(jmp_buf jbuf, templ_t * tpl, macro_t * mac, macro_t * end_mac);
 
-LOCAL void
+static void
 gen_new_block(templ_t * tpl);
 
-LOCAL macro_t *
-mLoad_Leave(templ_t * tpl, macro_t * mac, char const ** p_scan);
-
 /*
  *  Extracted from loadPseudo.c
  */
-LOCAL char const *
-do_suffix(char const * const pzData, char const * pzFileName, int lineNo);
+static char const *
+do_suffix(char const * const text, char const * fname, int lineNo);
 
-LOCAL char const *
-loadPseudoMacro(char const * pzData, char const * pzFileName);
+static char const *
+load_pseudo_mac(char const * text, char const * fname);
 
 /*
  *  Extracted from scmStrings.c
  */
-LOCAL void
+static void
 ag_scribble_init(void);
 
-LOCAL void
+static void
 ag_scribble_deinit(void);
 
-LOCAL void
+static void
 ag_scribble_free(void);
 
-LOCAL char *
+static char *
 ag_scribble(ssize_t size);
 
-LOCAL char *
+static char *
 ag_scm2zchars(SCM s, const char * type);
 
 /*
  *  Extracted from tpLoad.c
  */
-LOCAL templ_t *
+static templ_t *
 find_tpl(char const * pzTemplName);
 
-LOCAL tSuccess
+static tSuccess
 find_file(char const * in_name,
           char *       res_name,
           char const * const * sfx_list,
           char const * referring_tpl);
 
-LOCAL templ_t *
+static templ_t *
 tpl_load(char const * fname, char const * referrer);
 
-LOCAL void
+static void
 tpl_unload(templ_t * tpl);
 
-LOCAL void
+static void
 cleanup(templ_t * tpl);
 
 /*
  *  Extracted from tpParse.c
  */
-LOCAL macro_t *
+static macro_t *
 parse_tpl(macro_t * mac, char const ** p_scan);
 
 /*
  *  Extracted from tpProcess.c
  */
-LOCAL void
+static void
 gen_block(templ_t * tpl, macro_t * mac, macro_t * emac);
 
-LOCAL out_spec_t *
+static out_spec_t *
 next_out_spec(out_spec_t * os);
 
-LOCAL void
+static void
 process_tpl(templ_t * tpl);
 
-LOCAL void
+static void
 out_close(bool purge);
 
-#ifdef REDEF_LOCAL
-#  undef LOCAL
-#  define LOCAL
-#endif
 #endif /* AGEN5_PROTO_H_GUARD */
