@@ -1,13 +1,15 @@
+
 /**
- * \file gdinit.c
- *
- *  getdefs Copyright (c) 1999-2012 by Bruce Korb - all rights reserved
+ *  @file gdinit.c
+ *  @addtogroup columns
+ *  @{
+ */
+/*
+ *  getdefs Copyright (C) 1999-2013 by Bruce Korb - all rights reserved
  *
  *  Author:            Bruce Korb <bkorb@gnu.org>
- *  Time-stamp:        "2011-12-29 10:02:47 bkorb"
- *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2012 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (C) 1992-2013 by Bruce Korb - all rights reserved
  *
  *  AutoGen is free software: you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -95,7 +97,7 @@ compressOptionText(char* pzS, char* pzE)
     } compressDone:;
 
     {
-        size_t len = (pzD - pzR);
+        size_t len = (size_t)(pzD - pzR);
         pzD = malloc(len + 1);
         if (pzD == NULL)
             die("cannot dup %d byte string\n", (int)(pzD - pzR));
@@ -205,8 +207,8 @@ static void
 loadStdin(void)
 {
     char z[ 4096 ];
-    int    ct  = 0;
-    tCC**  ppz = STACKLST_OPT(INPUT);
+    int  ct  = 0;
+    char const ** ppz = STACKLST_OPT(INPUT);
 
     if (isatty(STDIN_FILENO)) {
         fputs("getdefs error:  no inputs were specified and stdin is a tty\n",
@@ -218,7 +220,7 @@ loadStdin(void)
         char* pz = z + strlen(z);
 
         if (pz[-1] != '\n') {
-            tSCC zErr[] =
+            static char const zErr[] =
                 "getdefs error: input line not newline terminated\n";
             fputs(zErr, stderr);
             exit(EXIT_FAILURE);
@@ -249,8 +251,8 @@ loadStdin(void)
 LOCAL void
 processEmbeddedOptions(char* pzText)
 {
-    tSCC zStStr[] = "/*=--";
-    tSCC zEndSt[] = "=*/";
+    static char const zStStr[] = "/*=--";
+    static char const zEndSt[] = "=*/";
 
     for (;;) {
         char* pzStart = strstr(pzText, zStStr);
@@ -272,7 +274,7 @@ processEmbeddedOptions(char* pzText)
         optionLoadLine(&getdefsOptions, pzStart);
 
         if (HAVE_OPT(SUBBLOCK) && (sblct != STACKCT_OPT(SUBBLOCK))) {
-            tCC** ppz = STACKLST_OPT(SUBBLOCK);
+            char const ** ppz = STACKLST_OPT(SUBBLOCK);
             ppz[ sblct ] = fixupSubblockString(ppz[sblct]);
         }
         pzText = pzEnd + sizeof(zEndSt);
@@ -326,13 +328,13 @@ set_define_re(void)
      */
     {
         char zRER[MAXNAMELEN];
-        static char const zReErr[] =
+        static char const regex_err[] =
             "Regex error %d (%s):  Cannot compile reg expr:\n\t%s\n";
 
         int rerr = regcomp(&define_re, def_pat, REG_EXTENDED | REG_ICASE);
         if (rerr != 0) {
             regerror(rerr, &define_re, zRER, sizeof(zRER));
-            die(zReErr, rerr, zRER, def_pat);
+            die(regex_err, rerr, zRER, def_pat);
         }
 
         if (free_pat)
@@ -341,7 +343,7 @@ set_define_re(void)
         rerr = regcomp(&attrib_re, zAttribRe, REG_EXTENDED | REG_ICASE);
         if (rerr != 0) {
             regerror(rerr, &attrib_re, zRER, sizeof(zRER));
-            die(zReErr, rerr, zRER, zAttribRe);
+            die(regex_err, rerr, zRER, zAttribRe);
         }
     }
 }
@@ -406,8 +408,8 @@ validateOptions(void)
      *  Prepare each sub-block entry so we can parse easily later.
      */
     if (HAVE_OPT(SUBBLOCK)) {
-        int     ct  = STACKCT_OPT( SUBBLOCK);
-        tCC**   ppz = STACKLST_OPT(SUBBLOCK);
+        int           ct  = STACKCT_OPT( SUBBLOCK);
+        char const ** ppz = STACKLST_OPT(SUBBLOCK);
 
         /*
          *  FOR each SUBBLOCK argument,
@@ -433,7 +435,7 @@ validateOptions(void)
      */
     if (  HAVE_OPT(ORDERING)
        && (OPT_ARG(ORDERING) != NULL)) {
-        tSCC zIndexPreamble[] =
+        static char const preamble[] =
             "# -*- buffer-read-only: t -*- vi: set ro:\n"
             "#\n# DO NOT EDIT THIS FILE - it is auto-edited by getdefs\n";
 
@@ -441,10 +443,10 @@ validateOptions(void)
         if (pzIndexText == NULL) {
             pzIndexText = pzEndIndex = pzIndexEOF = malloc((size_t)0x4000);
             indexAlloc = 0x4000;
-            pzEndIndex += sprintf(pzEndIndex, "%s", zIndexPreamble);
+            pzEndIndex += sprintf(pzEndIndex, "%s", preamble);
         } else {
-            pzEndIndex  = pzIndexEOF = pzIndexText + strlen(pzIndexText);
-            indexAlloc = (pzEndIndex - pzIndexText) + 1;
+            pzEndIndex = pzIndexEOF = pzIndexText + strlen(pzIndexText);
+            indexAlloc = (size_t)(pzEndIndex - pzIndexText) + 1;
         }
 
         /*
@@ -466,22 +468,26 @@ validateOptions(void)
     }
 
     {
-        tSCC zAgNameChars[] = "abcdefghijklmnopqrstuvwxyz"
-            "ABCDEFGHIJKLMNOPQRSTUVWXYZ" "0123456789" "_-^";
-        tSCC zUserNameChs[] = ":.$%*!~<>&@";
-        tCC* p = zAgNameChars;
+        static char const ag_nm_chars[] =
+            "abcdefghijklmnopqrstuvwxyz"
+            "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+            "0123456789" "_-^";
+        static char const usr_nm_chars[] = ":.$%*!~<>&@";
+
+        char const * p = ag_nm_chars;
 
         while (*p)
             zUserNameCh[(unsigned)(*p++)] = 3;
 
-        p = zUserNameChs;
+        p = usr_nm_chars;
         while (*p)
             zUserNameCh[(unsigned)(*p++)] = 1;
     }
 }
 
 
-/* emacs
+/** @}
+ *
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
